// SPDX-License-Identifier: GPL-3.0-or-later
//
// bdc_csv.h
//
// Copyright (C) 2011-2024, University of Vienna and Vienna Institute for Nature Conservation & Analyses, Andreas Gattringer.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//

#ifndef BDC_CSV_H_
#define BDC_CSV_H_

#include <stdint.h>
#include <stdio.h>

#include "bdc_strings/bdc_strings.h"

///@brief Data structure for simple csv files
struct bdc_csv;

struct bdc_csv *csv_read_file(FILE *input, int expected_fields);

struct bdc_csv *csv_from_filename(const char *filename, int expected_fields);

int32_t csv_row_count(const struct bdc_csv *csv);

int32_t csv_column_count(const struct bdc_csv *csv);

int32_t csv_get_column_idx(const struct bdc_csv *csv, const char *column_name);

double csv_get_double_value_from_column_index(const struct bdc_csv *csv, int32_t row, int32_t column_idx);

int32_t csv_get_int32_value_from_column_index(const struct bdc_csv *csv, int32_t row, int32_t column_idx);

double csv_get_double_from_column_name(const struct bdc_csv *csv, int32_t row, const char *field_name);

void csv_free(struct bdc_csv **csv);


#ifdef DEBUG_CSV_ENABLED
#define DBG_CSV(X) {X} while(0);
#else
#define DBG_CSV(X);
#endif

#endif





