// SPDX-License-Identifier: GPL-3.0-or-later
//
// bdc_hashtable.h
//
// Copyright (C) 2011-2024, University of Vienna and Vienna Institute for Nature Conservation & Analyses, Andreas Gattringer.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//

#ifndef BDC_HASHTABLE_H
#define BDC_HASHTABLE_H

#include <stdint.h>
#include <stddef.h>
#include <stdbool.h>

#define MIN_SIZE 1024


enum bdc_value_type {
        BDC_STRING = 0,
        BDC_INT64 = 1,
        BDC_DOUBLE = 2,
        BDC_CUSTOM
};




struct bdc_hash_table;
struct bdc_value;

bool remove_key(struct bdc_hash_table *table, const char *key);
struct bdc_item *intern_bdc_add_entry(struct bdc_hash_table *table, const char *key, struct bdc_value *value);

struct bdc_item *bdc_ht_add_string(struct bdc_hash_table *table, const char *key, const char *value);

struct bdc_item *bdc_ht_add_custom(struct bdc_hash_table *table, const char *key, void *custom);

struct bdc_item *bdc_ht_add_int64(struct bdc_hash_table *table, const char *key, int64_t value);

struct bdc_item *bdc_ht_add_double(struct bdc_hash_table *table, const char *key, double value);

__attribute__((unused)) struct bdc_hash_table *new_hash_table(void);

__attribute__((unused)) void print_hash_table(const struct bdc_hash_table *table);

struct bdc_value *bdc_get_value_from_key(struct bdc_hash_table *table, const char *key);

__attribute__((unused)) bool bdc_key_in_hash_table(struct bdc_hash_table *table, const char *key);

bool bdc_ht_get_int64_from_key(struct bdc_hash_table *table, const char *key, int64_t *result);

bool bdc_ht_get_string_from_key(struct bdc_hash_table *table, const char *key, char **result);

__attribute__((unused)) bool bdc_ht_get_double_from_key(struct bdc_hash_table *table, const char *key, double *result);
void destroy_hash_table(struct bdc_hash_table **ht);

#define bdc_ht_get_data_from_key(TABLE, KEY, RESULT) _GENERIC((RESULT), \
        int64_t *: bdc_ht_get_int64_from_key,                           \
        char **:   bdc_ht_get_string_from_key,                          \
        double *:  bdc_ht_get_value_from_key,                           \
)(TABLE, KEY, (RESULT))

#define bdc_ht_add_entry(table, key, value) \
   _Generic((value),\
        int64_t: bdc_ht_add_int64,    \
        int: bdc_ht_add_int64,              \
        double: bdc_ht_add_double,          \
        void *:  bdc_ht_add_custom, \
        char *: bdc_ht_add_string)(table, key, value)


// move to private later // FIXME
void rehash_table(struct bdc_hash_table *table);
#endif //BDC_HASHTABLE_H
