// SPDX-License-Identifier: GPL-3.0-or-later
//
// bdc_io.h
//
// Copyright (C) 2011-2024, University of Vienna and Vienna Institute for Nature Conservation & Analyses, Andreas Gattringer.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//

#ifndef BDC_IO_H
#define BDC_IO_H

#define MAX_STRING_BUFF_LENGTH 4096
#if defined(WIN32) || defined (_WIN32)
#define DIRECTORY_SEPARATOR "\\"
#else
#define DIRECTORY_SEPARATOR "/"
#endif


#include "bdc_strings/bdc_strings.h"

char *slurp_file(const char *filename);
char *read_single_line(FILE *file);

#define ENSURE_FILE_OPENED(FILE_HANDLE, FILENAME)\
if (! (FILE_HANDLE))\
{\
        fprintf(stderr, "ERROR: Couldn't open file %s in %s: %d (%s)\n", FILENAME, __FILE__, __LINE__, __func__);\
        exit(EXIT_FAILURE);\
}

char *assemble_path(const struct string_array *path);

char *assemble_filename(struct string_array *path, struct string_array *filename, char *field_separator, char *extension);

bool check_and_create_directory_if_needed(const struct string_array *directory_path);

bool directory_exists(const char *path);

bool directory_writable(const char *path);

void make_directory(const char *path);
#endif //BDC_IO_H
