// SPDX-License-Identifier: GPL-3.0-or-later
//
// bdc_memory.h
//
// Copyright (C) 2011-2024, University of Vienna and Vienna Institute for Nature Conservation & Analyses, Andreas Gattringer.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//

#ifndef bdc_memory_H
#define bdc_memory_H

#include <stddef.h> // for size_t
#include <stdbool.h>
#include <stdint.h>

void *malloc_or_die_trace(size_t size, const char *func);

void *realloc_or_die(void *ptr, size_t size);

void *malloc_or_die(size_t size);

void *calloc_or_die(size_t count, size_t size);


struct array_dimension {
        int32_t rows;
        int32_t cols;
};

struct cats_2d_array_float {
        float **data;
        struct array_dimension dimension;
};

struct cats_2d_array_char {
        char **data;
        struct array_dimension dimension;
};

struct cats_2d_array_double {
        double **data;
        struct array_dimension dimension;
};

struct cats_2d_array_char *new_2d_array_char(const struct array_dimension *dimension);

struct cats_2d_array_double *new_2d_array_double(const struct array_dimension *dimension);

__attribute__((unused)) struct cats_2d_array_float *new_2d_array_float(struct array_dimension *dimension);

void cleanup_2d_array_char(struct cats_2d_array_char **grid);

void cleanup_2d_array_float(struct cats_2d_array_float **grid);

void cleanup_2d_array_double(struct cats_2d_array_double **grid);

#define free_cats_grid(PTR_TO_GRID) _Generic((PTR_TO_GRID), \
 struct cats_2d_array_float **:        cleanup_2d_array_float, \
 struct cats_2d_array_double **:       cleanup_2d_array_double, \
 struct cats_2d_array_char **:         cleanup_2d_array_char \
 )(PTR_TO_GRID)

bool array_dimension_valid(const struct array_dimension *dimension);

__attribute__((unused)) bool array_dimensions_match(const struct array_dimension *d1, const struct array_dimension *d2);

void *new_raw_2d_array_from_dimension(const struct array_dimension *dimension, size_t member_size);


void cleanup_raw_2d_double_array(double ***data, int64_t rows);

void cleanup_raw_2d_float_array(float ***data, int64_t rows);

void cleanup_raw_2d_int32t_array(int32_t ***data, int64_t rows);

void cleanup_raw_2d_char_array(char ***data, int64_t rows);

void cleanup_raw_2d_uchar_array(unsigned char ***data, int64_t rows);

void zero_raw_1d_float_array(float *array, size_t size);

void zero_raw_1d_long_double_array(long double *array, size_t size);

void zero_raw_1d_int_array(int *array, size_t size);

#define free_grid(PTR_TO_GRID, ROWS) _Generic((PTR_TO_GRID), \
 double ***:        cleanup_raw_2d_double_array, \
 float ***:         cleanup_raw_2d_float_array, \
 int32_t ***:       cleanup_raw_2d_int32t_array, \
 char ***:          cleanup_raw_2d_char_array, \
 unsigned char ***: cleanup_raw_2d_uchar_array\
 )(PTR_TO_GRID, (ROWS))


#define zero_array(ARRAY, ENTRIES) _Generic((ARRAY), \
float *:         zero_raw_1d_float_array, \
long double *:   zero_raw_1d_long_double_array, \
int *:           zero_raw_1d_int_array\
)(ARRAY, (ENTRIES))

void *new_raw_2d_array(size_t rows, size_t cols, size_t size);

void *new_raw_1d_array(size_t count, size_t size);

#endif //bdc_memory_H
