// SPDX-License-Identifier: GPL-3.0-or-later
//
// bdc_timer.c
//
// Copyright (C) 2011-2024, University of Vienna and Vienna Institute for Nature Conservation & Analyses, Andreas Gattringer.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//

#include <assert.h>
#include <stdlib.h>

#include "bdc_memory/bdc_memory.h"
#include "bdc_time.h"



struct timeval *time_difference(struct timeval *start, struct timeval *end)
{
        struct timeval *diff = malloc_or_die(sizeof(struct timeval));

        if (start->tv_sec == end->tv_sec) {
                diff->tv_sec = 0;
                diff->tv_usec = end->tv_usec - start->tv_usec;
        } else {
                diff->tv_usec = 1000000 - start->tv_usec;
                diff->tv_sec = end->tv_sec - (start->tv_sec + 1);
                diff->tv_usec += end->tv_usec;

                if (diff->tv_usec >= 1000000) {
                        diff->tv_usec -= 1000000;
                        diff->tv_sec += 1;
                }
        }

        return diff;
}


struct bdc_timer start_new_timer(void)
{
        struct bdc_timer t = {0};
        t.start_clock = clock();
        get_time(&t.start_timeval);
        return t;
}


void start_timer(struct bdc_timer *t)
{
        assert(t != NULL);
        t->start_clock = clock();
        get_time(&t->start_timeval);
}


void stop_timer(struct bdc_timer *t)
{
        assert(t != NULL);
        clock_t now = clock();

        struct timeval time_now;
        double elapsed_time;
        get_time(&time_now);
        elapsed_time = ((double) (now - t->start_clock) / CLOCKS_PER_SEC);

        t->time_normal = elapsed_time;

        double difference_real;
        struct timeval *difference;

        difference = time_difference(&t->start_timeval, &time_now);
        difference_real = (double) difference->tv_sec + (double) difference->tv_usec / (1000.0 * 1000.0);
        t->time_real = difference_real;

        free(difference);
}
